/*
 * $Id: libbridge_if.c,v 1.4 2000/01/22 09:39:57 root Exp $
 *
 * Copyright (C) 2000 Lennert Buytenhek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include "libbridge.h"

struct bridge *br_find_bridge(char *brname)
{
	struct bridge *b;

	b = bridge_list;
	while (b != NULL) {
		if (!strcmp(b->ifname, brname))
			return b;

		b = b->next;
	}

	return NULL;
}

struct port *br_find_port(struct bridge *br, char *portname)
{
	char index;
	struct port *p;

	if (!(index = if_nametoindex(portname)))
		return NULL;

	p = br->firstport;
	while (p != NULL) {
		if (p->ifindex == index)
			return p;

		p = p->next;
	}

	return NULL;
}

int br_ioctl(unsigned long arg0, unsigned long arg1, unsigned long arg2)
{
	unsigned long arg[3];

	arg[0] = arg0;
	arg[1] = arg1;
	arg[2] = arg2;

	return ioctl(br_socket_fd, SIOCGIFBR, arg);
}

void br_add_bridge(char *brname)
{
	char _br[IFNAMSIZ];

	memcpy(_br, brname, IFNAMSIZ);
	if (br_ioctl(BRCTL_ADD_BRIDGE, (unsigned long)_br, 0) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EEXIST:
		fprintf(stderr,	"device %s already exists; can't create "
			"bridge with the same name\n", brname);
		break;

	default:
		perror("br_add_bridge");
		exit(-1);
	}
}

void br_del_bridge(char *brname)
{
	char _br[IFNAMSIZ];

	memcpy(_br, brname, IFNAMSIZ);
	if (br_ioctl(BRCTL_DEL_BRIDGE, (unsigned long)_br, 0) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case ENXIO:
		fprintf(stderr, "bridge %s doesn't exist; can't delete it\n", brname);
		break;

	case EBUSY:
		fprintf(stderr, "bridge %s is still up; can't delete it\n", brname);
		break;

	default:
		perror("br_del_bridge");
		exit(-1);
	}
}

int br_device_ioctl(struct bridge *br, unsigned long arg0, unsigned long arg1, unsigned long arg2, unsigned long arg3)
{
	unsigned long args[4];
	struct ifreq ifr;

	args[0] = arg0;
	args[1] = arg1;
	args[2] = arg2;
	args[3] = arg3;

	memcpy(ifr.ifr_name, br->ifname, IFNAMSIZ);
	((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;

	return ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr);
}

void br_add_interface(struct bridge *br, int ifindex)
{
	char ifname[IFNAMSIZ];

	if (br_device_ioctl(br, BRCTL_ADD_IF, ifindex, 0, 0) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EBUSY:
		fprintf(stderr,	"device %s is already a member of a bridge; "
			"can't enslave it to bridge %s.\n",
			if_indextoname(ifindex, ifname), br->ifname);
		break;

	default:
		perror("br_add_interface");
		exit(-1);
	}
}

void br_del_interface(struct bridge *br, int ifindex)
{
	char ifname[IFNAMSIZ];

	if (br_device_ioctl(br, BRCTL_DEL_IF, ifindex, 0, 0) >= 0)
		return;

	switch (errno) {
	case 0:
		break;

	case EINVAL:
		fprintf(stderr, "device %s is not a slave of %s\n",
			if_indextoname(ifindex, ifname), br->ifname);
		break;

	default:
		perror("br_del_interface");
		exit(-1);
	}
}

void br_set_bridge_forward_delay(struct bridge *br, int forward_delay)
{
	if (br_device_ioctl(br, BRCTL_SET_BRIDGE_FORWARD_DELAY, forward_delay, 0, 0) < 0) {
		perror("br_set_bridge_forward_delay");
		return;
	}
}

void br_set_bridge_hello_time(struct bridge *br, int hello_time)
{
	if (br_device_ioctl(br, BRCTL_SET_BRIDGE_HELLO_TIME, hello_time, 0, 0) < 0) {
		perror("br_set_bridge_hello_time");
		return;
	}
}

void br_set_bridge_max_age(struct bridge *br, int max_age)
{
	if (br_device_ioctl(br, BRCTL_SET_BRIDGE_MAX_AGE, max_age, 0, 0) < 0) {
		perror("br_set_bridge_max_age");
		return;
	}
}

void br_set_ageing_time(struct bridge *br, int ageing_time)
{
	if (br_device_ioctl(br, BRCTL_SET_AGEING_TIME, ageing_time, 0, 0) < 0) {
		perror("br_set_ageing_time");
		return;
	}
}

void br_set_gc_interval(struct bridge *br, int gc_interval)
{
	if (br_device_ioctl(br, BRCTL_SET_GC_INTERVAL, gc_interval, 0, 0) < 0) {
		perror("br_set_gc_interval");
		return;
	}
}

void br_set_stp_state(struct bridge *br, int stp_state)
{
	if (br_device_ioctl(br, BRCTL_SET_BRIDGE_STP_STATE, stp_state, 0, 0) < 0) {
		perror("br_set_stp_state");
		return;
	}
}

void br_set_bridge_priority(struct bridge *br, int bridge_priority)
{
	if (br_device_ioctl(br, BRCTL_SET_BRIDGE_PRIORITY, bridge_priority, 0, 0) < 0) {
		perror("br_set_bridge_priority");
		return;
	}
}

void br_set_port_priority(struct port *p, int port_priority)
{
	if (br_device_ioctl(p->parent, BRCTL_SET_PORT_PRIORITY, p->index, port_priority, 0) < 0) {
		perror("br_set_port_priority");
		return;
	}
}

void br_set_path_cost(struct port *p, int path_cost)
{
	if (br_device_ioctl(p->parent, BRCTL_SET_PATH_COST, p->index, path_cost, 0) < 0) {
		perror("br_set_path_cost");
		return;
	}
}

int br_read_fdb(struct bridge *br, struct fdb_entry *fdbs, int offset, int num)
{
	int err;

	if ((err = br_device_ioctl(br, BRCTL_GET_FDB_ENTRIES, (unsigned long)fdbs, num, offset)) < 0) {
		perror("br_read_fdb");
		exit(-1);
	}

	return err;
}
